@@ -10,7 +10,8 @@ term2table <- function(rowterm, colterm, env, n) {
     rowargs <- factors(rowterm)
     colargs <- factors(colterm)
     allargs <- c(rowargs, colargs)
-    subset <- NULL
+    rowsubset <- TRUE
+    colsubset <- TRUE
     values <- NULL
     summary <- NULL
     format <- NA
@@ -22,7 +23,22 @@ term2table <- function(rowterm, colterm, env, n) {
             format <- e[[2]]
         else if (fn == "Justify") 
             justification <- as.character(e[[if (length(e) > 2) 3 else 2]])
-    	else if (fn != "Heading" && !identical(e, 1)) {
+        else if (fn == "Percent") {
+            env1 <- new.env(parent=env)
+            percent <- function(x, y) 100*length(x)/length(y)
+            env1$Percent <- function(denom="all", fn=percent) {
+              if (is.null(summary)) {
+                if (identical(denom, "all")) summary <<- function(x) fn(x, values)
+                else if (identical(denom, "row")) summary <<- function(x) fn(x, values[rowsubset])
+                else if (identical(denom, "col")) summary <<- function(x) fn(x, values[colsubset])
+                else if (is.logical(denom)) summary <<- function(x) fn(x, values[denom])
+                else summary <<- function(x) fn(x, denom)
+                summaryname <<- "Percent"
+              } else
+    	        stop("Summary fn not allowed with Percent")
+            }
+            eval(e, env1)
+        } else if (fn != "Heading" && !identical(e, 1)) {
     	    arg <- eval(e, env)
     	    asis <- inherits(arg, "AsIs")
     	    if (asis || is.vector(arg)) {
@@ -32,10 +48,11 @@ term2table <- function(rowterm, colterm, env, n) {
     	    }
     	    
     	    if (!asis && is.logical(arg)) {
-    	    	if (is.null(subset))
-    	    	    subset <- arg
+    	    	arg <- arg & !is.na(arg)
+    	    	if (i <= length(rowargs))
+    	    	    rowsubset <- rowsubset & arg
     	    	else
-    	    	    subset <- subset & arg
+    	    	    colsubset <- colsubset & arg
     	    } else if (asis || is.atomic(arg)) {
     	    	if (is.null(values)) {
     	    	    values <- arg
@@ -59,11 +76,40 @@ term2table <- function(rowterm, colterm, env, n) {
     	     "indeterminate")    
     if (is.null(summary)) summary <- length
     if (is.null(values)) values <- rep(NA, n)
-    if (is.null(subset)) subset <- TRUE 
+    subset <- rowsubset & colsubset 
     structure(list(summary(values[subset])), n=n, format=format, 
                    justification=justification)
 }
 
+# This moves column names into their own column
+moveColnames <- function(labels, do_move = (names != "")) {
+    attrs <- attributes(labels)
+    names <- colnames(labels)
+    colnamejust <- attrs$colnamejust
+    justification <- attrs$justification
+    for (i in rev(seq_along(do_move))) {
+    	if (do_move[i]) {
+	    before <- seq_len(i-1)
+	    after <- seq_len(ncol(labels) - i + 1) + i - 1
+    	    labels <- cbind(labels[,before,drop=FALSE], "", labels[,after,drop=FALSE])
+	    labels[1,i] <- names[i]
+	    names <- c(names[before], "", "", names[after][-1])
+	    if (length(colnamejust)) {
+	    	colnamejust <- c(colnamejust[before], NA_character_, colnamejust[after])
+	    }
+	    if (length(justification)) 
+	    	justification <- cbind(justification[,before,drop=FALSE], 
+	    	    NA_character_, justification[,after,drop=FALSE])
+	}
+    }
+    attrs$colnamejust <- colnamejust
+    attrs$justification <- justification
+    attrs$dim <- dim(labels)
+    attrs$dimnames[[2]] <- names
+    attributes(labels) <- attrs
+    labels
+}
+    	
 getLabels <- function(e, rows=TRUE, justify=NA, head=NULL, suppress=0) {
     op <- ""
     justification <- NULL
@@ -71,7 +117,7 @@ getLabels <- function(e, rows=TRUE, justify=NA, head=NULL, suppress=0) {
     Heading <- head
     result <- if (rows) matrix(NA, ncol=0, nrow=1)
     	      else      matrix(NA, ncol=1, nrow=0)
-    if (is.call(e) && (op <- as.character(e[[1]])) %in% c("*", "+") ) {
+    if (is.call(e) && (op <- as.character(e[[1]])) == "*")  {
         leftLabels <- getLabels(e[[2]], rows, justify, head, suppress)
 	nrl <- nrow(leftLabels)
 	ncl <- ncol(leftLabels)
@@ -81,12 +127,10 @@ getLabels <- function(e, rows=TRUE, justify=NA, head=NULL, suppress=0) {
 	leftjustify <- attr(leftLabels, "justify")
 	Heading <- attr(leftLabels, "Heading")
 	leftsuppress <- attr(leftLabels, "suppress")
-	if (op == "*") {
-	    righthead <- Heading
-	    suppress <- leftsuppress
-	    if (!is.null(leftjustify))
-	    	justify <- leftjustify
-	} else righthead <- head   
+	righthead <- Heading
+	suppress <- leftsuppress
+	if (!is.null(leftjustify))
+	    justify <- leftjustify
 	leftjustification <- attr(leftLabels, "justification")
 	leftcolnamejust <- attr(leftLabels, "colnamejust")
 
@@ -96,129 +140,199 @@ getLabels <- function(e, rows=TRUE, justify=NA, head=NULL, suppress=0) {
 	rightjustify <- attr(rightLabels, "justify")
 	Heading <- attr(rightLabels, "Heading")
 	suppress <- attr(rightLabels, "suppress")
-	if (op == "*") {
-	    if (!is.null(rightjustify))
-		justify <- rightjustify
-	}
+	if (!is.null(rightjustify))
+	    justify <- rightjustify
 	rightjustification <- attr(rightLabels, "justification")
 	rightcolnamejust <- attr(rightLabels, "colnamejust")
-    	switch(op,
-        "*" = {
-            if (rows) {
-		result <- justification <- matrix(NA_character_, nrl*nrr, ncl + ncr)
-		colnames(result) <- c(colnames(leftLabels), colnames(rightLabels))
-		colnamejust <- c(leftcolnamejust, rightcolnamejust)
-		if (!is.null(head)) {
-		    if (ncr > 0) {
-		        colnames(result)[ncl + 1] <- head
-		        colnamejust[ncl + 1] <- leftjustify
-		    } else if (is.null(Heading))
-		    	Heading <- head
-		}
-		for (i in seq_len(nrl)) {
-		    j <- 1 + (i-1)*nrr
-		    k <- seq_len(ncl)
-		    result[j, k] <- leftLabels[i,]
-		    if (!is.null(leftjustification))
-		    	justification[j, k] <- leftjustification[i,]
-		    j <- (i-1)*nrr + seq_len(nrr)
-		    k <- ncl+seq_len(ncr)
-		    result[j, k] <- rightLabels
-		    if (!is.null(rightjustification))
-		    	justification[j, k] <- rightjustification
-		}
-	    } else {
-		result <- justification <- matrix(NA_character_, nrl + nrr, ncl*ncr)
-		for (i in seq_len(ncl)) {
-		    j <- seq_len(nrl)
-		    k <- 1 + (i-1)*ncr
-		    result[j, k] <- leftLabels[,i]
-		    if (!is.null(leftjustification))
-		    	justification[j,k] <- leftjustification[,i]
-		    j <- nrl+seq_len(nrr)
-		    k <- (i-1)*ncr + seq_len(ncr)
-		    result[j, k] <- rightLabels
-		    if (!is.null(rightjustification))
-		    	justification[j,k] <- rightjustification
-		}
-	    }
-        },
-        "+" = {
-            if (rows) {
-		cols <- max(ncl, ncr)
-		result <- matrix("", nrl + nrr, cols)
-		justification <- matrix(NA_character_, nrl + nrr, cols)
-		colnames(result) <- colnames(leftLabels)
-		colnamejust <- leftcolnamejust
-		j <- seq_len(nrl)
-		k <- (cols-ncl) + seq_len(ncl)
-		result[j, k] <- leftLabels
-		if (!is.null(leftjustification))
-		    justification[j, k] <- leftjustification
-		j <- nrl+seq_len(nrr)
-		k <- (cols-ncr) + seq_len(ncr)
-		result[j, k] <- rightLabels
-		if (!is.null(rightjustification))
-		    justification[j, k] <- rightjustification 
-	    } else {
-		nrows <- max(nrl, nrr)
-		result <- matrix("", nrows, ncl + ncr)
-		justification <- matrix(NA_character_, nrows, ncl + ncr)
-		j <- (nrows-nrl) + seq_len(nrl)
+
+	if (rows) {
+	    result <- justification <- matrix(NA_character_, nrl*nrr, ncl + ncr)
+	    colnames(result) <- c(colnames(leftLabels), colnames(rightLabels))
+	    colnamejust <- c(leftcolnamejust, rightcolnamejust)
+	    for (i in seq_len(nrl)) {
+		j <- 1 + (i-1)*nrr
 		k <- seq_len(ncl)
-		result[j, k] <- leftLabels
+		result[j, k] <- leftLabels[i,]
 		if (!is.null(leftjustification))
-		    justification[j, k] <- leftjustification
-		j <- (nrows-nrr) + seq_len(nrr)
+		    justification[j, k] <- leftjustification[i,]
+		j <- (i-1)*nrr + seq_len(nrr)
 		k <- ncl+seq_len(ncr)
-		result[j,k] <- rightLabels
+		result[j, k] <- rightLabels
 		if (!is.null(rightjustification))
 		    justification[j, k] <- rightjustification
 	    }
-        })
-    } else if (op == "(") {
-    	result <- getLabels(e[[2]], rows, justify, NULL, suppress)
-    	if (!is.null(head)) {
-	    if (suppress == 0) {
-		saveattrs <- attributes(result)
-		if (rows) {
-		    result <- cbind("", result)
-		    justification <- cbind(justify, justification)
-		} else {
-		    result <- rbind("", result)
-		    justification <- rbind(justify, justification)
-		}
+	} else {
+	    result <- justification <- matrix(NA_character_, nrl + nrr, ncl*ncr)
+	    for (i in seq_len(ncl)) {
+		j <- seq_len(nrl)
+		k <- 1 + (i-1)*ncr
+		result[j, k] <- leftLabels[,i]
+		if (!is.null(leftjustification))
+		    justification[j,k] <- leftjustification[,i]
+		j <- nrl+seq_len(nrr)
+		k <- (i-1)*ncr + seq_len(ncr)
+		result[j, k] <- rightLabels
+		if (!is.null(rightjustification))
+		    justification[j,k] <- rightjustification
+	    }
+	}
+    } else if (op == "+") {
+        leftLabels <- getLabels(e[[2]], rows, justify, NULL, suppress)
+	nrl <- nrow(leftLabels)
+	ncl <- ncol(leftLabels)
+	# Heading and justify are the settings to carry on to later terms
+	# justification is the matrix of justifications for
+	# each label
+	leftjustify <- attr(leftLabels, "justify")
+	Heading <- attr(leftLabels, "Heading")
+	leftsuppress <- attr(leftLabels, "suppress")
+	leftjustification <- attr(leftLabels, "justification")
+	leftcolnamejust <- attr(leftLabels, "colnamejust")
+
+	rightLabels <- getLabels(e[[3]], rows, justify, NULL, suppress)
+	nrr <- nrow(rightLabels)
+	ncr <- ncol(rightLabels)
+	rightjustify <- attr(rightLabels, "justify")
+	Heading <- attr(rightLabels, "Heading")
+	suppress <- attr(rightLabels, "suppress")
+
+	rightjustification <- attr(rightLabels, "justification")
+	rightcolnamejust <- attr(rightLabels, "colnamejust")
+
+	if (rows) {
+	    # Here we have a problem:  we need to stack two things, each of which
+	    # may have column names.  We use the following rule:
+	    #  - if the column names for both things match, just use them.
+	    #  - if the left one has a name, and the right doesn't, use the left name
+	    #  - if both have names that don't match, add them as extra column(s)
+	    #  - if the right has a name, and the left doesn't, treat as unmatched names
+	    
+	    leftnames <- colnames(leftLabels)
+	    if (is.null(leftnames)) leftnames <- rep("", ncl)
+	    rightnames <- colnames(rightLabels)
+	    if (is.null(rightnames)) rightnames <- rep("", ncr)
+	    if (!identical(rightnames, rep("", ncr)) &&
+	        !identical(leftnames, rightnames)) {
+	        rightLabels <- moveColnames(rightLabels)
+	        leftLabels <- moveColnames(leftLabels)
+	        ncr <- ncol(rightLabels)
+	        ncl <- ncol(leftLabels)
+	        rightnames <- rep("", ncr)
+	        leftnames <- rep("", ncl)
+	    }    
+	    cols <- max(ncl, ncr)
+	    # Pad all to same width
+	    if (ncl < ncr) {
+	        pad <- rep("", ncr-ncl)
+	    	leftnames <- c(pad, leftnames)
+	    	if (length(leftcolnamejust)) {
+	    	  pad[TRUE] <- NA_character_
+	    	  leftcolnamejust <- c(pad, leftcolnamejust)
+	    	}
+	    	pad <- matrix("", nrl, ncr-ncl)
+	    	leftLabels <- cbind(pad, leftLabels)
+	    	if (!is.null(leftjustification)) {
+	    	  pad[TRUE] <- NA_character_
+	    	  leftjustification <- cbind(pad, leftjustification)
+	    	}
+	    	ncl <- ncr
+	    } else if (ncl > ncr) {
+	        pad <- rep("", ncl-ncr)
+	    	rightnames <- c(pad, rightnames)
+	    	if (length(rightcolnamejust)) {
+	    	  pad[TRUE] <- NA_character_
+	    	  rightcolnamejust <- c(pad, rightcolnamejust)
+	    	}
+	    	pad <- matrix("", nrr, ncl-ncr)
+	    	rightLabels <- cbind(pad, rightLabels)
+	    	if (!is.null(rightjustification)) {
+	    	  pad[TRUE] <- NA_character_
+	    	  rightjustification <- cbind(pad, rightjustification)
+	    	}
+	    	ncr <- ncl
+	    }
+	    result <- matrix("", nrl + nrr, cols)
+	    justification <- matrix(NA_character_, nrl + nrr, cols)
+	    colnames <- rep("", cols)
+	    colnamejust <- rep(NA_character_, cols)
+	    j <- seq_len(nrl)
+	    result[j, ] <- leftLabels
+	    colnames <- leftnames
+	    if (length(leftcolnamejust))
+	        colnamejust <- leftcolnamejust	    
+	    if (length(leftjustification))
+		justification[j, ] <- leftjustification
+
+	    j <- nrl+seq_len(nrr)
+	    result[j, ] <- rightLabels
+	    
+	    if (!is.null(rightjustification))
+		justification[j, ] <- rightjustification 
+	    if (!is.null(head)) {
+		colnames[1] <- head
+		head <- NULL
+	    }
+	    colnames(result) <- colnames
+	} else {
+	    nrows <- max(nrl, nrr)
+	    result <- matrix("", nrows, ncl + ncr)
+	    justification <- matrix(NA_character_, nrows, ncl + ncr)
+	    j <- (nrows-nrl) + seq_len(nrl)
+	    k <- seq_len(ncl)
+	    result[j, k] <- leftLabels
+	    if (!is.null(leftjustification))
+		justification[j, k] <- leftjustification
+	    j <- (nrows-nrr) + seq_len(nrr)
+	    k <- ncl+seq_len(ncr)
+	    result[j,k] <- rightLabels
+	    if (!is.null(rightjustification))
+		justification[j, k] <- rightjustification
+	    if (!is.null(head)) {
+		result <- rbind(rep(NA_character_, ncol(result)),
+				result)
 		result[1,1] <- head
-	    } else 
-		suppress <- suppress - 1
-        } 	
+		justification <- rbind(justification[1,], justification)
+	    }
+	}
+    } else if (op == "(") {
+    	return(getLabels(e[[2]], rows, justify, head, suppress))
     } else if (op == ".Format") {
     	result <- if (rows) matrix(NA, ncol=0, nrow=1)
     		  else      matrix(NA, ncol=1, nrow=0)
     } else if (op == "Heading") {
     	if (length(e) > 1) {
-    	    if (suppress <= 0) {
+    	    override <- TRUE
+    	    if (length(e) > 2) {
+    	    	override <- as.logical(e[[3]])
+    	    	if (is.na(override))
+    	    	    stop("Second argument in ", deparse(e), " must be logical",
+    	    	         call. = FALSE)
+    	    }
+    	    if (suppress <= 0 && (is.null(Heading) || override)) {
     	    	if (is.character(e[[2]]))
     	    	    Heading <- e[[2]]
     	    	else
-    	    	    Heading=deparse(e[[2]])
+    	    	    Heading <- deparse(e[[2]])
     	    } else 
     	    	suppress <- suppress - 1
     	} else
     	    suppress <- suppress + 1
     } else if (op == "Justify") {
     	justify <- as.character(e[[2]])
-    } else if (suppress > 0)  # The rest just add a single label; suppress it
+    } else if (suppress > 0) {  # The rest just add a single label; suppress it
     	suppress <- suppress - 1
-    else if (!is.null(head)) {
+    } else if (!is.null(head)) {
     	result <- matrix(head, 1,1, dimnames=list(NULL, ""))
     	Heading <- NULL
+    } else if (op == "Percent") {
+        result <- matrix("Percent", 1,1, dimnames=list(NULL, ""))
     } else if (identical(e, 1)) 
     	result <- matrix("All", 1,1, dimnames=list(NULL, ""))
     else
     	result <- matrix(deparse(e), 1,1, dimnames=list(NULL, ""))
     if (is.null(justification))
     	justification <- matrix(justify, nrow(result), ncol(result))
+    stopifnot(identical(dim(result), dim(justification)))
     structure(result, justification = justification, 
     	colnamejust = colnamejust, justify = justify,
    	Heading = Heading, suppress = suppress)
@@ -231,7 +345,8 @@ expandExpressions <- function(e, env) {
 	    if (length(e) > 2)
 		e[[3]] <- expandExpressions(e[[3]], env)
 	} else if (op == "Format" || op == ".Format" 
-	        || op == "Heading" || op == "Justify")
+	        || op == "Heading" || op == "Justify"
+	        || op == "Percent")
 	    e
 	else {
 	    v <- eval(e, envir=env)
@@ -274,24 +389,21 @@ expandFactors <- function(e, env) {
     	call(op, expandFactors(e[[2]],env),expandFactors(e[[3]],env))
     else if (op == "(")
     	expandFactors(e[[2]],env)
-    else if (op == "=")
-    	call("*", call("Heading", as.name(deparse(e[[2]]))),
-    		  expandFactors(e[[3]], env))
-    else if (op == ".Format" || op == "Heading" || op == "Justify")
+    else if (op == "=") {
+    	rhs <- expandFactors(e[[3]], env)
+    	if (is.call(rhs) && as.character(rhs[[1]]) == "*"
+    	 && is.call(rhs[[2]]) && as.character(rhs[[c(2,1)]]) == "Heading") {
+    	    rhs[[c(2,2)]] <- as.name(deparse(e[[2]]))
+    	    rhs
+    	} else
+    	    call("*", call("Heading", as.name(deparse(e[[2]]))), rhs)
+    } else if (op == ".Format" || op == "Heading" || 
+             op == "Justify" || op == "Percent")
     	e
     else {
     	v <- eval(e, envir=env)
-    	if (is.factor(v) & !inherits(v, "AsIs")) {
-    	    levs <- levels(v)
-    	    if (length(levs)) {
-    	     	f <- call("*", call("Heading", as.name(levs[1])), 
-    	     		       call("==", e, levs[1]))
-    	     	for (i in seq_along(levs)[-1]) 
-    	     	    f <- call("+", f, call("*", call("Heading", as.name(levs[i])), 
-    	     	                                call("==", e, levs[i])))
-    	     	e <- call("*", call("Heading", as.name(deparse(e))), f)
-    	     }
-    	}
+    	if (is.factor(v) & !inherits(v, "AsIs")) 
+    	    e <- Factor(v, expr=e, override=FALSE)
     	e
     }
 }
@@ -389,7 +501,7 @@ tabular.formula <- function(table, data=NULL, n, suppressLabels=0, ...) {
     	    justification <- attr(term, "justification")
     	    row <- cbind(row, term)
     	    rowformats <- cbind(rowformats, format)
-    	    rowjustification <- cbind(rowjustification, justification) 
+    	    rowjustification <- cbind(rowjustification, justification)
     	}
     	result <- rbind(result, row)
     	formats <- rbind(formats, rowformats)
